﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 6.7.2016
 * Time: 10.24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;

namespace Pshais.Indicators
{
	
	public class DynamicBuySell : IIndicatorScript
	{
		double _timeFrameHours = 4;
		double _marginBuy = 0.25;
		double _marginSell = 0.25;
		
		double _highest = 0;
		double _lowest = 0;
		double _buyLevel = 0;
		double _sellLevel = 0;
		
		int _interval = 1;
		
		int pointsInFrame = 1;
		
		TicTacTec.TA.Library.Core.RetCode sarReturnCode;
		#region IIndicatorScript implementation
		public void Init()
		{
			
		}
		public IndicatorResult GetResult(IndicatorContext context)
		{			
			try
            {
				_interval = context.Interval;
				
                IPriceInstrument instrument = context.PriceInstrument; // Its pre-created for speed based on the "TicksBack" property field

                // Lets do the calculation
                int position = instrument.Close.Count() - (1 + context.Offset); // minus 1 because of counting from 0 + account for the backtest
                
                
                
                //context.Logger.LogToFile(string.Format("context.Interval: {0}\npointsInFrame: {1}", context.Interval, pointsInFrame));
			
                var dates = context.PriceInstrument.Timestamps;
                var highs = context.PriceInstrument.High;
                var lows = context.PriceInstrument.Low;
                var opens = context.PriceInstrument.Open;
                var closes = context.PriceInstrument.Close;
                
                var topBid = context.PriceTick.Bid;
                var topAsk = context.PriceTick.Ask;
				
				var outputs = calculate(highs, lows, context);
				
				_highest = outputs[0];
				_lowest = outputs[1];
				_sellLevel = outputs[2];
				_buyLevel = outputs[3];
				
				var curClose = closes[position];
				var curOpen = opens[position];
				var efficiency = _sellLevel / _buyLevel - 1;
				var tradeEfficiency = topAsk / topBid - 1;
				
				if( efficiency > 0.000 )
				{
					if( curClose <= _buyLevel )
					{
						return IndicatorResult.BuyLong;
					}
					
					if( curClose >= _sellLevel )
					{
						return IndicatorResult.SellShort;
					}
				}

            }
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex));
            }

            return IndicatorResult.StayNoPosition; // Default output
		}
		
		
		List<double> calculate(double[] highs, double[] lows, IndicatorContext context)
		{
			// calculate points in time-frame 
			pointsInFrame = (int)((_timeFrameHours*60) / context.Interval);
			int position = highs.Length - (1 + context.Offset);
			int firstIndex = 0, i = 0;
			
			// clamp input data count
			var clamped_highs = highs.Skip(position - pointsInFrame).Take(pointsInFrame).ToArray();
			var clamped_lows = lows.Skip(position - pointsInFrame).Take(pointsInFrame).ToArray();
			
			var highestHigh = clamped_highs.Max();
			var lowestLow = clamped_lows.Min();
			var difference = highestHigh - lowestLow;
			
			var mSell = highestHigh - (difference * _marginSell);
			var mBuy = lowestLow + (difference * _marginBuy);
						
			return new List<double>{
				highestHigh,
				lowestLow,
				mSell,
				mBuy
			};
		}
		
		
		public List<ScriptParameter> GetParameters()
		{
            var rtn = new List<ScriptParameter>
            {
                new ScriptParameter("Time-frame", ScriptParameterType.Decimal, _timeFrameHours.ToString("F1")),
                new ScriptParameter("Buy margin", ScriptParameterType.Decimal, _marginBuy.ToString("F2")),
                new ScriptParameter("Sell margin", ScriptParameterType.Decimal, _marginSell.ToString("F2"))
            };

            return rtn;
		}
		public void SetParameters(Dictionary<string, object> parameters)
		{
			foreach (var item in parameters) // lets just loop them all
            {
				switch( item.Key )
				{
					case "Time-frame":
						_timeFrameHours = ParseDouble(item.Value);
						break;
						
					case "Buy margin":
						_marginBuy = ParseDouble(item.Value);
						break;
						
					case "Sell margin":
						_marginSell = ParseDouble(item.Value);
						break;
				}
            }
		}
		public List<ChartRecord> GetChartLines()
		{
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "Highest", HexLineColor = "#880000" },
                new ChartRecord() { ChartIndex = 0, Name = "Sell level", HexLineColor = "#FF0000" },
                new ChartRecord() { ChartIndex = 0, Name = "Buy level", HexLineColor = "#00FF00" },
                new ChartRecord() { ChartIndex = 0, Name = "Lowest", HexLineColor = "#008800" }
            };

            return chartLines;
		}
		public List<decimal> GetChartData()
		{
            var chartData = new List<decimal>
            {
                (decimal)_highest,
                (decimal)_sellLevel,
                (decimal)_buyLevel,
                (decimal)_lowest
            };

            return chartData;
		}
		public string Name {
			get {
				return "pshai's Dynamic Buy/Sell";
			}
		}
		public int TicksBack {
			get {
				return (int)((_timeFrameHours*60) / _interval);
			}
		}
		#endregion
		
		double ParseDouble(object obj)
		{
			var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			double output;
			try {
				double.TryParse(
					obj.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
			}
			catch
			{
				output = 0;
			}
			
			return output;
		}
	}
}